/***************************************************************************
 *   Copyright (C) 2005 by Borko Bokovi                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <ctype.h>

#include "bitboard.h"
#include "piece.h"
#include "square.h"
#include "move.h"
#include "generator.h"
#include "uci.h"

UINT64 squareKeys[14][64];
UINT64 playerKey;

#define uChangePlayer(position) {position->player ^= 1;position->opponent ^= 1;}
#define changePlayer(position) {uChangePlayer(position);position->key ^= playerKey;}

inline int isInCheck(struct bitboard* position){
	int square = indexOfFirstBit64(&position->king[position->player]);
	if(attackRook(position,square) &
		(position->rooks[position->opponent]|position->queens[position->opponent])) return 1;
	if(attackBishop(position,square) & 
		(position->bishops[position->opponent]|position->queens[position->opponent])) return 1;
	if(attackKnight[square]&position->knights[position->opponent]) return 1;
	if(attackPawn[position->player][square]&position->pawns[position->opponent]) return 1;
	return 0;
}

inline int isIllegal(struct bitboard* position){
	int square = indexOfFirstBit64(&position->king[position->opponent]);
	if(attackRook(position,square)&(position->rooks[position->player]|position->queens[position->player]))
		return 1;
	if(attackBishop(position,square)&(position->bishops[position->player]|position->queens[position->player]))
		return 1;
	if(attackKnight[square]&position->knights[position->player]) return 1;
	if(attackPawn[position->opponent][square]&position->pawns[position->player]) return 1;
	if(attackKing[square]&position->king[position->player]) return 1;
        return 0;
}

inline int isAttackedByBlackE1F1G1(struct bitboard* position){
	if( (attackRook(position,e1) | attackRook(position,f1) | attackRook(position,g1) )&
		(position->rooks[BLACK] | position->queens[BLACK]) ) return 1;
	if( (attackBishop(position,e1) | attackBishop(position,f1) | attackBishop(position,g1))&
		(position->bishops[BLACK] | position->queens[BLACK]) ) return 1;
	if(16309248ull&position->knights[BLACK]) return 1;
	if(63488ull&position->pawns[BLACK]) return 1;
	if(63736ull&position->king[BLACK]) return 1;
	return 0;
}

inline int isAttackedByBlackC1D1E1(struct bitboard* position){
	if( (attackRook(position,c1) | attackRook(position,d1) | attackRook(position,e1) )&
		(position->rooks[BLACK] | position->queens[BLACK]) ) return 1;
	if( (attackBishop(position,c1) | attackBishop(position,d1) | attackBishop(position,e1))&
		(position->bishops[BLACK] | position->queens[BLACK]) ) return 1;
	if(4093696ull&position->knights[BLACK]) return 1;
	if(15872ull&position->pawns[BLACK]) return 1;
	if(15934ull&position->king[BLACK]) return 1;
	return 0;
}

inline int isAttackedByWhiteE8F8G8(struct bitboard* position){
	if( (attackRook(position,e8) | attackRook(position,f8) | attackRook(position,g8) )&
		(position->rooks[WHITE] | position->queens[WHITE]) ) return 1;
	if( (attackBishop(position,e8) | attackBishop(position,f8) | attackBishop(position,g8))&
		(position->bishops[WHITE] | position->queens[WHITE]) ) return 1;
	if(62197173760032768ull&position->knights[WHITE]) return 1;
	if(69805794224242688ull&position->pawns[WHITE]) return 1;
	if(17940089115630370816ull&position->king[WHITE]) return 1;
	return 0;
}

inline int isAttackedByWhiteC8D8E8(struct bitboard* position){
	if( (attackRook(position,c8) | attackRook(position,d8) | attackRook(position,e8) )&
		(position->rooks[WHITE] | position->queens[WHITE]) ) return 1;
	if( (attackBishop(position,c8) | attackBishop(position,d8) | attackBishop(position,e8))&
		(position->bishops[WHITE] | position->queens[WHITE]) ) return 1;
	if(33563691949490176ull&position->knights[WHITE]) return 1;
	if(17451448556060672ull&position->pawns[WHITE]) return 1;
	if(4485022278907592704ull&position->king[WHITE]) return 1;
	return 0;
}

void initBitboard(UINT64 aSquaresKeys[14][64], UINT64 aPlayerkey){
	int i,j;
	if(aSquaresKeys == 0){
		for(i=0; i<14; i++) for(j=0; j<64; j++) squareKeys[i][j] = rand64();
		playerKey = rand64();
	}
	else{
		for(i=0; i<14; i++) for(j=0; j<64; j++) squareKeys[i][j] = aSquaresKeys[i][j];
		playerKey = aPlayerkey;
	}
}

void bitboardEmptyPosition(struct bitboard* position){
	int i;
	position->pawns = &position->pieces[PAWN];
	position->knights = &position->pieces[KNIGHT];
	position->bishops = &position->pieces[BISHOP];
	position->rooks = &position->pieces[ROOK];
	position->queens = &position->pieces[QUEEN];
	position->king = &position->pieces[KING];
	for(i=WHITE; i<=BLACK; i++){
		position->pawns[i] = position->knights[i] = position->bishops[i] = 0;
		position->rooks[i] = position->queens[i] = position->king[i] = 0;
		position->pieces[i] = 0;
		position->material[i] = position->numberOfPieces[i] = 0;
	}
	position->key = position->piecesRL90 = position->piecesRL45 = position->piecesRR45 = 0;
	position->ply = position->player = position->enPassant = position->castling = 0;
	position->check = 0;
	position->opponent = 1;
	position->notFreeSquares = 0;
	position->freeSquares = ~0;
	for(i=0; i<MAX_PLY; i++){
		position->historyOfKeys[i] = 0;
		position->staticMoves[i] = position->historyOfMoves[i] = 0;
	}
	for(i=0; i<64; i++) position->types[i] = EMPTY;
}

inline void addPiece(struct bitboard* position, int piece, int square){
	UINT64 bbsquare = ONE64 << square;
	int player = piece%2;
	position->material[player] += pieceValue[piece];
	position->numberOfPieces[player] ++;
	position->pieces[player] ^= bbsquare;
	position->pieces[piece] ^= bbsquare;
	position->notFreeSquares ^= bbsquare;			
	position->freeSquares = ~position->notFreeSquares;	
	position->piecesRL90 ^= ONE64 << rotateL90[square];	
	position->piecesRL45 ^= ONE64 << rotateL45[square];
	position->piecesRR45 ^= ONE64 << rotateR45[square];
	position->key ^= squareKeys[piece][square];		
	position->types[square] = piece;
}

inline void removePiece(struct bitboard* position, int piece, int square){
	UINT64 bbsquare = ONE64 << square;								
	int player = piece % 2;
	position->material[player] -= pieceValue[piece];
	position->numberOfPieces[player] --;
	position->pieces[player] ^= bbsquare;
	position->pieces[piece] ^= bbsquare;
	position->notFreeSquares ^= bbsquare;							
	position->freeSquares = ~position->notFreeSquares;				
	position->piecesRL90 ^= ONE64 << rotateL90[square];				
	position->piecesRL45 ^= ONE64 << rotateL45[square];
	position->piecesRR45 ^= ONE64 << rotateR45[square];
	position->key ^= squareKeys[piece][square];						
	position->types[square] = EMPTY;
}

inline void movePiece(struct bitboard* position, int piece, int from, int to){
	UINT64 bbsquare = ONE64 << from | ONE64 << to;					
	position->pieces[piece%2] ^= bbsquare;
	position->pieces[piece] ^= bbsquare;
	position->notFreeSquares ^= bbsquare;							
	position->freeSquares = ~position->notFreeSquares;				
	position->piecesRL90 ^= ONE64 << rotateL90[from] | ONE64 << rotateL90[to]; 
	position->piecesRL45 ^= ONE64 << rotateL45[from] | ONE64 << rotateL45[to];
	position->piecesRR45 ^= ONE64 << rotateR45[from] | ONE64 << rotateR45[to];
	position->key ^= squareKeys[piece][from] ^ squareKeys[piece][to];
	position->types[from] = EMPTY;
	position->types[to] = piece;
}

inline void uMovePiece(struct bitboard* position, int piece, int from, int to){
	UINT64 bbsquare = ONE64 << from | ONE64 << to;				
	position->pieces[piece%2] ^= bbsquare;
	position->pieces[piece] ^= bbsquare;
	position->notFreeSquares ^= bbsquare;						
	position->freeSquares = ~position->notFreeSquares;			
	position->piecesRL90 ^= ONE64 << rotateL90[from] | ONE64 << rotateL90[to]; 
	position->piecesRL45 ^= ONE64 << rotateL45[from] | ONE64 << rotateL45[to];
	position->piecesRR45 ^= ONE64 << rotateR45[from] | ONE64 << rotateR45[to];
	position->types[from] = piece;
	position->types[to] = EMPTY;
}

inline void moveCapturePiece(struct bitboard* position, int piece, int cpiece, int from, int to){
	UINT64 bbsquare1 = ONE64 << to;								
	UINT64 bbsquare2 = bbsquare1 | ONE64 << from;				
	position->pieces[position->opponent] ^= bbsquare1;			
	position->pieces[position->player] ^= bbsquare2;			
	position->pieces[cpiece] ^= bbsquare1;						
	position->pieces[piece] ^= bbsquare2;						
	position->notFreeSquares ^= ONE64 << from;					
	position->freeSquares = ~position->notFreeSquares;			
	position->piecesRL90 ^= ONE64 << rotateL90[from];			
	position->piecesRL45 ^= ONE64 << rotateL45[from];
	position->piecesRR45 ^= ONE64 << rotateR45[from];
	position->types[from] = EMPTY;								
	position->types[to] = piece;
	position->numberOfPieces[position->opponent] --;			
	position->material[position->opponent] -= pieceValue[cpiece];	
	position->key ^= squareKeys[piece][from] ^ squareKeys[piece][to] ^ squareKeys[cpiece][to]; 
}

inline void uMoveCapturePiece(struct bitboard* position, int piece, int cpiece, int from, int to){
	UINT64 bbsquare1 = ONE64 << to;								
	UINT64 bbsquare2 = bbsquare1 | ONE64 << from;				
	position->pieces[position->opponent] ^= bbsquare1;			
	position->pieces[position->player] ^= bbsquare2;			
	position->pieces[cpiece] ^= bbsquare1;						
	position->pieces[piece] ^= bbsquare2;						
	position->notFreeSquares ^= ONE64 << from;					
	position->freeSquares = ~position->notFreeSquares;			
	position->piecesRL90 ^= ONE64 << rotateL90[from];			
	position->piecesRL45 ^= ONE64 << rotateL45[from];
	position->piecesRR45 ^= ONE64 << rotateR45[from];
	position->types[from] = piece;								
	position->types[to] = cpiece;
	position->numberOfPieces[position->opponent]++;				
	position->material[position->opponent] += pieceValue[cpiece];	
}

inline void moveWEnPassantPiece(struct bitboard* position, int from, int to){
	UINT64 bbsquare1 = ONE64 << from | ONE64 << to;					
	int bpi = to-8;
	UINT64 bbsquare2 = ONE64 << bpi;								
	position->pieces[WHITE] ^= bbsquare1;
	position->pieces[BLACK] ^= bbsquare2;
	position->pieces[WHITE_PAWN] ^= bbsquare1;
	position->pieces[BLACK_PAWN] ^= bbsquare2;
	position->notFreeSquares ^= bbsquare1 ^ bbsquare2;				
	position->freeSquares = ~position->notFreeSquares;				
	position->piecesRL90 ^= ONE64 << rotateL90[from] | ONE64 << rotateL90[to] | ONE64 << rotateL90[bpi]; 
	position->piecesRL45 ^= ONE64 << rotateL45[from] | ONE64 << rotateL45[to] | ONE64 << rotateL45[bpi];
	position->piecesRR45 ^= ONE64 << rotateR45[from] | ONE64 << rotateR45[to] | ONE64 << rotateR45[bpi];
	position->key ^= squareKeys[WHITE_PAWN][from] ^ squareKeys[WHITE_PAWN][to] ^ squareKeys[BLACK_PAWN][bpi];
	position->numberOfPieces[BLACK] --;
	position->material[BLACK] -= PAWN_VALUE;
	position->types[from] = EMPTY;
	position->types[to] = WHITE_PAWN;
	position->types[bpi] = EMPTY;
}

inline void uMoveWEnPassantPiece(struct bitboard* position, int from, int to){
	UINT64 bbsquare1 = ONE64 << from | ONE64 << to;					
	int bpi = to-8;
	UINT64 bbsquare2 = ONE64 << bpi;								
	position->pieces[WHITE] ^= bbsquare1;
	position->pieces[BLACK] ^= bbsquare2;
	position->pieces[WHITE_PAWN] ^= bbsquare1;
	position->pieces[BLACK_PAWN] ^= bbsquare2;
	position->notFreeSquares ^= bbsquare1 ^ bbsquare2;				
	position->freeSquares = ~position->notFreeSquares;				
	position->piecesRL90 ^= ONE64 << rotateL90[from] | ONE64 << rotateL90[to] | ONE64 << rotateL90[bpi]; 
	position->piecesRL45 ^= ONE64 << rotateL45[from] | ONE64 << rotateL45[to] | ONE64 << rotateL45[bpi];
	position->piecesRR45 ^= ONE64 << rotateR45[from] | ONE64 << rotateR45[to] | ONE64 << rotateR45[bpi];
	position->numberOfPieces[BLACK] ++;
	position->material[BLACK] += PAWN_VALUE;
	position->types[from] = WHITE_PAWN;
	position->types[to] = EMPTY;
	position->types[bpi] = BLACK_PAWN;
}

inline void moveBEnPassantPiece(struct bitboard* position, int from, int to){
	UINT64 bbsquare1 = ONE64 << from | ONE64 << to;					
	int wpi = to+8;
	UINT64 bbsquare2 = ONE64 << wpi;								
	position->pieces[BLACK] ^= bbsquare1;
	position->pieces[WHITE] ^= bbsquare2;
	position->pieces[BLACK_PAWN] ^= bbsquare1;
	position->pieces[WHITE_PAWN] ^= bbsquare2;
	position->notFreeSquares ^= bbsquare1 ^ bbsquare2;				
	position->freeSquares = ~position->notFreeSquares;				
	position->piecesRL90 ^= ONE64 << rotateL90[from] | ONE64 << rotateL90[to] | ONE64 << rotateL90[wpi]; 
	position->piecesRL45 ^= ONE64 << rotateL45[from] | ONE64 << rotateL45[to] | ONE64 << rotateL45[wpi];
	position->piecesRR45 ^= ONE64 << rotateR45[from] | ONE64 << rotateR45[to] | ONE64 << rotateR45[wpi];
	position->key ^= squareKeys[BLACK_PAWN][from] ^ squareKeys[BLACK_PAWN][to] ^ squareKeys[WHITE_PAWN][wpi];
	position->numberOfPieces[WHITE] --;
	position->material[WHITE] -= PAWN_VALUE;
	position->types[from] = EMPTY;
	position->types[to] = BLACK_PAWN;
	position->types[wpi] = EMPTY;
}

inline void uMoveBEnPassantPiece(struct bitboard* position, int from, int to){
	UINT64 bbsquare1 = ONE64 << from | ONE64 << to;					
	int wpi = to+8;
	UINT64 bbsquare2 = ONE64 << wpi;								
	position->pieces[BLACK] ^= bbsquare1;
	position->pieces[WHITE] ^= bbsquare2;
	position->pieces[BLACK_PAWN] ^= bbsquare1;
	position->pieces[WHITE_PAWN] ^= bbsquare2;
	position->notFreeSquares ^= bbsquare1 ^ bbsquare2;				
	position->freeSquares = ~position->notFreeSquares;				
	position->piecesRL90 ^= ONE64 << rotateL90[from] | ONE64 << rotateL90[to] | ONE64 << rotateL90[wpi]; 
	position->piecesRL45 ^= ONE64 << rotateL45[from] | ONE64 << rotateL45[to] | ONE64 << rotateL45[wpi];
	position->piecesRR45 ^= ONE64 << rotateR45[from] | ONE64 << rotateR45[to] | ONE64 << rotateR45[wpi];
	position->numberOfPieces[WHITE] ++;
	position->material[WHITE] += PAWN_VALUE;
	position->types[from] = BLACK_PAWN;
	position->types[to] = EMPTY;
	position->types[wpi] = WHITE_PAWN;
}

void bitboardStartPosition(struct bitboard* position){
	int i;
	bitboardEmptyPosition(position);
	for (i = a2; i <=h2; i++) addPiece(position, WHITE_PAWN, i);
	addPiece(position, WHITE_ROOK, a1);
	addPiece(position, WHITE_ROOK, h1);
	addPiece(position, WHITE_KNIGHT, b1);
	addPiece(position, WHITE_KNIGHT, g1);
	addPiece(position, WHITE_BISHOP, c1);
	addPiece(position, WHITE_BISHOP, f1);
	addPiece(position, WHITE_QUEEN, d1);
	addPiece(position, WHITE_KING, e1);
	for (i = a7; i <=h7; i++) addPiece(position, BLACK_PAWN, i);
	addPiece(position, BLACK_ROOK, a8);
	addPiece(position, BLACK_ROOK, h8);
	addPiece(position, BLACK_KNIGHT, b8);
	addPiece(position, BLACK_KNIGHT, g8);
	addPiece(position, BLACK_BISHOP, c8);
	addPiece(position, BLACK_BISHOP, f8);
	addPiece(position, BLACK_QUEEN, d8);
	addPiece(position, BLACK_KING, e8);
	position->player = WHITE;
	position->castling = 15;
}

void bitboardSetPosition(struct bitboard* position, char fen[]){
	int index = 0, line = 7, square = 56, intc;
	char token[50];
	bitboardEmptyPosition(position);
	uciNextToken(fen, token);
	while(token[index] != '\0'){
		intc = (int)token[index];
		if(intc >= 48 && intc <=59){ square += intc - 48;}
		else if(token[index] != '/'){ addPiece(position,charToPiece(token[index]),square++); }
		else{ square = --line * 8; }
		index ++;
	}
	uciNextToken(fen,token);
	if(token[0] == 'b') changePlayer(position);
	uciNextToken(fen,token);
	index = 0;
	while(token[index] != '\0'){
		switch (token[index]){
			case 'K':{ position->castling |= 1; break; }
			case 'Q':{ position->castling |= 2; break; }
			case 'k':{ position->castling |= 4; break; }
			case 'q':{ position->castling |= 8; break; }
		}
		index++;
	}
	uciNextToken(fen,token);
	if(token[0] != '-') position->enPassant = squareToInt(token);
	position->check = isInCheck(position);
}

void makeNormalMove(struct bitboard* position, int move){
	int from, to, piece;
	from = moveGetFrom(move);
	to = moveGetTo(move);
	piece = position->types[from];
	movePiece(position,piece,from,to);
	switch(piece){
		case WHITE_PAWN:{
			position->staticMoves[position->ply+1] = 1;
			if(to-from == 16 && (attackPawn[WHITE][to-8] & position->pawns[BLACK]) != 0){
				position->enPassant = to-8;
				position->key ^= position->enPassant;
			}
			break;
		}
		case BLACK_PAWN:{
			position->staticMoves[position->ply+1] = 1;
			if(from-to == 16 && (attackPawn[BLACK][from-8] & position->pawns[WHITE]) !=0){
				position->enPassant = from-8;
				position->key ^= position->enPassant;
			}
			break;
		}
		case WHITE_ROOK:{
			position->staticMoves[position->ply+1] = position->staticMoves[position->ply]+1;
			if(position->castling & 3){
				if(position->castling & 1 && from == h1){
					position->castling ^= 1;
					position->key ^= 1;
				}
				if(position->castling & 2 && from == a1){
					position->castling ^= 2;
					position->key ^= 2;
				}
			}
			break;
		}
		case BLACK_ROOK:{
			position->staticMoves[position->ply+1] = position->staticMoves[position->ply]+1;
			if(position->castling & 12){
				if(position->castling & 4 && from == h8){
					position->castling ^= 4;
					position->key ^= 4;
				}
				if(position->castling & 8 && from == a8){
					position->castling ^= 8;
					position->key ^= 8;
				}
			}
			break;
		}
		case WHITE_KING:{
			position->staticMoves[position->ply+1] = position->staticMoves[position->ply]+1;
			if(position->castling & 3){
				position->key ^= position->castling & 3;
				position->castling &= 12;
			}
			break;
		}
		case BLACK_KING:{
			position->staticMoves[position->ply+1] = position->staticMoves[position->ply]+1;
			if(position->castling & 12){
				position->key ^= position->castling & 12;
				position->castling &= 3;
			}
			break;
		}
		default:{
			position->staticMoves[position->ply+1] = position->staticMoves[position->ply]+1;
			break;
		}
	}	
}

void makeCaptureMove(struct bitboard* position, int piece, int cPiece, int from, int to){
	moveCapturePiece(position, piece, cPiece, from, to);
	position->staticMoves[position->ply+1] = 1;
	if(position->castling){
		switch(from){
			case h1:{
				if(position->castling & 1){
					position->castling ^= 1;
					position->key ^= 1;
				}
				break;
			}
			case a1:{
				if(position->castling & 2){
					position->castling ^= 2;
					position->key ^= 2;
				}
				break;
			}
			case h8:{
				if(position->castling & 4){
					position->castling ^= 4;
					position->key ^= 4;
				}
				break;
			}
			case a8:{
				if(position->castling & 8){
					position->castling ^= 8;
					position->key ^= 8;
				}
				break;
			}
			case e1:{
				if(position->castling & 3){
					position->key ^= position->castling & 3;
					position->castling ^= position->castling & 3;
				}
				break;
			}
			case e8:{
				if(position->castling & 12){
					position->key ^= position->castling & 12;
					position->castling ^= position->castling & 12;
				}
				break;
			}
		}
		switch(to){
			case h1:{
				if(position->castling & 1){
					position->castling ^= 1;
					position->key ^= 1;
				}
				break;
			}
			case a1:{
				if(position->castling & 2){
					position->castling ^= 2;
					position->key ^= 2;
				}
				break;
			}
			case h8:{
				if(position->castling & 4){
					position->castling ^= 4;
					position->key ^= 4;
				}
				break;
			}
			case a8:{
				if(position->castling & 8){
					position->castling ^= 8;
					position->key ^= 8;
				}
				break;
			}
		}
	}	
}

int makeSMove(struct bitboard* position, char move[]){
	move[4] = tolower(move[4]);
	int moves[GENERATOR_MOVES], i;
	int size = generateMoves(position,moves);
	char smove[6];
	for(i=0; i<size; i++){
		moveSPrint(smove, moves[i]);
		if(!strcmp(smove,move)) return makeMove(position,moves[i]);
	}
	return 0;
}

void makeSMoves(struct bitboard* position, char moves[]){
	char token[6];
	while(1){
		uciNextToken(moves,token);
		if(!strcmp("",token)) break;
		if(!makeSMove(position,token)){	
			bitboardPrint(position);
			printf("Illegal move (makeSMoves)(%s)!!!\n",token);
			 exit(1);
		}
  	}
}

int makeMove(struct bitboard* position, int move){
	position->historyOfKeys[position->ply] = position->key;
	position->key ^= position->enPassant;	
	moveSetCastling(move,position->castling);
	moveSetEnPassant(move,position->enPassant);
	moveSetCheck(move,position->check);
	position->enPassant = 0;
	switch(moveGetType(move)){
		case NORMAL_MOVE:{
			makeNormalMove(position,move);
			break;
		}
		case CAPTURE_MOVE:{
			int from, to, piece, cPiece;
			from = moveGetFrom(move);
			to = moveGetTo(move);
			piece = position->types[from];
			cPiece = position->types[to];
			moveSetCapturePiece(move,cPiece);
			makeCaptureMove(position, piece, cPiece, from,  to);
			break;
		}
		case ENPASSANT_MOVE:{
			position->staticMoves[position->ply+1] = 1;
			int to =  moveGetTo(move);
			if(to >32) moveWEnPassantPiece(position,moveGetFrom(move),to);
			else moveBEnPassantPiece(position,moveGetFrom(move),to);
			break;
		}
		case moveGetType(WHITE_CASTLING_KING_SIDE):{
			if(isAttackedByBlackE1F1G1(position)){
				position->enPassant = moveGetEnPassant(move);
				position->key ^= position->enPassant;
				return 0;
			}
			movePiece(position,WHITE_KING,e1,g1);
			movePiece(position,WHITE_ROOK,h1,f1);
			position->staticMoves[position->ply+1] = 1;
			position->castling &= 12;
			break;
		}
		case moveGetType(WHITE_CASTLING_QUEEN_SIDE):{
			if(isAttackedByBlackC1D1E1(position)){
				position->enPassant = moveGetEnPassant(move);
				position->key ^= position->enPassant;
				return 0;
			}
			movePiece(position,WHITE_KING,e1,c1);
			movePiece(position,WHITE_ROOK,a1,d1);
			position->staticMoves[position->ply+1] = 1;
			position->castling &= 12;
			break;
		}
		case moveGetType(BLACK_CASTLING_KING_SIDE):{
			if(isAttackedByWhiteE8F8G8(position)){
				position->enPassant = moveGetEnPassant(move);
				position->key ^= position->enPassant;
				return 0;
			}
			movePiece(position,BLACK_KING,e8,g8);
			movePiece(position,BLACK_ROOK,h8,f8);
			position->staticMoves[position->ply+1] = 1;
			position->castling &= 3;
			break;
		}
		case moveGetType(BLACK_CASTLING_QUEEN_SIDE):{
			if(isAttackedByWhiteC8D8E8(position)){
				position->enPassant = moveGetEnPassant(move);
				position->key ^= position->enPassant;
				return 0;
			}
			movePiece(position,BLACK_KING,e8,c8);
			movePiece(position,BLACK_ROOK,a8,d8);
			position->staticMoves[position->ply+1] = 1;
			position->castling &= 3;
			break;
		}
		case PROMOTION_QUEEN:{
			position->staticMoves[position->ply+1] = 1;
			int from = moveGetFrom(move);
			int to = moveGetTo(move);
			int cPiece = position->types[to];
			if(cPiece != EMPTY){
				moveSetCapturePiece(move,cPiece);
				removePiece(position,cPiece,to);
			}
			if(from > 32){
				removePiece(position,WHITE_PAWN,from);
				addPiece(position,WHITE_QUEEN,to);
				if(position->castling & 12){
					if((position->castling & 4) && to == h8)
						position->castling ^= 4;
					if((position->castling & 8) && to == a8)
						position->castling ^= 8;
				}
			}
			else{
				removePiece(position,BLACK_PAWN,from);
				addPiece(position,BLACK_QUEEN,to);
				if(position->castling & 3){
					if((position->castling & 1) && to == h1)
						position->castling ^= 1;
					if((position->castling & 2) && to == a1)
						position->castling ^= 2;
				}
			}
			break;
		}
		case PROMOTION_ROOK:{
			position->staticMoves[position->ply+1] = 1;
			int from = moveGetFrom(move);
			int to = moveGetTo(move);
			int cPiece = position->types[to];
			if(cPiece != EMPTY){
				moveSetCapturePiece(move,cPiece);
				removePiece(position,cPiece,to);
			}
			if(from > 32){
				removePiece(position,WHITE_PAWN,from);
				addPiece(position,WHITE_ROOK,to);
				if(position->castling & 12){
					if((position->castling & 4) && to == h8)
						position->castling ^= 4;
					if((position->castling & 8) && to == a8)
						position->castling ^= 8;
				}
			}
			else{
				removePiece(position,BLACK_PAWN,from);
				addPiece(position,BLACK_ROOK,to);
				if(position->castling & 3){
					if((position->castling & 1) && to == h1)
						position->castling ^= 1;
					if((position->castling & 2) && to == a1)
						position->castling ^= 2;
				}
			}
			break;
		}
		case PROMOTION_BISHOP:{
			position->staticMoves[position->ply+1] = 1;
			int from = moveGetFrom(move);
			int to = moveGetTo(move);
			int cPiece = position->types[to];
			if(cPiece != EMPTY){
				moveSetCapturePiece(move,cPiece);
				removePiece(position,cPiece,to);
			}
			if(from > 32){
				removePiece(position,WHITE_PAWN,from);
				addPiece(position,WHITE_BISHOP,to);
				if(position->castling & 12){
					if((position->castling & 4) && to == h8)
						position->castling ^= 4;
					if((position->castling & 8) && to == a8)
						position->castling ^= 8;
				}
			}
			else{
				removePiece(position,BLACK_PAWN,from);
				addPiece(position,BLACK_BISHOP,to);
				if(position->castling & 3){
					if((position->castling & 1) && to == h1)
						position->castling ^= 1;
					if((position->castling & 2) && to == a1)
						position->castling ^= 2;
				}
			}
			break;
		}
		case PROMOTION_KNIGHT:{
			position->staticMoves[position->ply+1] = 1;
			int from = moveGetFrom(move);
			int to = moveGetTo(move);
			int cPiece = position->types[to];
			if(cPiece != EMPTY){
				moveSetCapturePiece(move,cPiece);
				removePiece(position,cPiece,to);
			}
			if(from > 32){
				removePiece(position,WHITE_PAWN,from);
				addPiece(position,WHITE_KNIGHT,to);
				if(position->castling & 12){
					if((position->castling & 4) && to == h8)
						position->castling ^= 4;
					if((position->castling & 8) && to == a8)
						position->castling ^= 8;
				}
			}
			else{
				removePiece(position,BLACK_PAWN,from);
				addPiece(position,BLACK_KNIGHT,to);
				if(position->castling & 3){
					if((position->castling & 1) && to == h1)
						position->castling ^= 1;
					if((position->castling & 2) && to == a1)
						position->castling ^= 2;
				}
			}
			break;
		}
	}
	changePlayer(position);
	position->historyOfMoves[position->ply] = move;
	position->ply ++;
	if(isIllegal(position)){
		unmakeMove(position);
		return 0;
	}
	position->check = isInCheck(position);
	return 1;
}

int unmakeMove(struct bitboard* position){
	position->ply --;
	int move = position->historyOfMoves[position->ply];
	uChangePlayer(position);
	switch(moveGetType(move)){
		case NORMAL_MOVE:{
			int to = moveGetTo(move);
			uMovePiece(position, position->types[to], moveGetFrom(move), to);
			break;
		}
		case CAPTURE_MOVE:{
			int to = moveGetTo(move);
			int from = moveGetFrom(move);
			uMoveCapturePiece(position,position->types[to],moveGetCapturePiece(move), from, to);
			break;
		}
		case ENPASSANT_MOVE:{
			int to =  moveGetTo(move);
			if(to >32) uMoveWEnPassantPiece(position,moveGetFrom(move),to);
			else uMoveBEnPassantPiece(position,moveGetFrom(move),to);
			break;
		}
		case moveGetType(WHITE_CASTLING_KING_SIDE):{
			uMovePiece(position,WHITE_KING,e1,g1);
			uMovePiece(position,WHITE_ROOK,h1,f1);
			break;
		}
		case moveGetType(WHITE_CASTLING_QUEEN_SIDE):{
			uMovePiece(position,WHITE_KING,e1,c1);
			uMovePiece(position,WHITE_ROOK,a1,d1);
			break;
		}
		case moveGetType(BLACK_CASTLING_KING_SIDE):{
			uMovePiece(position,BLACK_KING,e8,g8);
			uMovePiece(position,BLACK_ROOK,h8,f8);
			break;
		}
		case moveGetType(BLACK_CASTLING_QUEEN_SIDE):{
			uMovePiece(position,BLACK_KING,e8,c8);
			uMovePiece(position,BLACK_ROOK,a8,d8);
			break;
		}
		case PROMOTION_QUEEN:{
			int from = moveGetFrom(move);
			int to = moveGetTo(move);
			int cPiece = moveGetCapturePiece(move);
			if(from > 32){
				addPiece(position,WHITE_PAWN,from);
				removePiece(position,WHITE_QUEEN,to);
			}
			else{
				addPiece(position,BLACK_PAWN,from);
				removePiece(position,BLACK_QUEEN,to);
			}
			if(cPiece != EMPTY) addPiece(position,cPiece,to);
			break;
		}
		case PROMOTION_ROOK:{
			int from = moveGetFrom(move);
			int to = moveGetTo(move);
			int cPiece = moveGetCapturePiece(move);
			if(from > 32){
				addPiece(position,WHITE_PAWN,from);
				removePiece(position,WHITE_ROOK,to);
			}
			else{
				addPiece(position,BLACK_PAWN,from);
				removePiece(position,BLACK_ROOK,to);
			}
			if(cPiece != EMPTY) addPiece(position,cPiece,to);
			break;
		}
		case PROMOTION_BISHOP:{
			int from = moveGetFrom(move);
			int to = moveGetTo(move);
			int cPiece = moveGetCapturePiece(move);
			if(from > 32){
				addPiece(position,WHITE_PAWN,from);
				removePiece(position,WHITE_BISHOP,to);
			}
			else{
				addPiece(position,BLACK_PAWN,from);
				removePiece(position,BLACK_BISHOP,to);
			}
			if(cPiece != EMPTY) addPiece(position,cPiece,to);
			break;
		}
		case PROMOTION_KNIGHT:{
			int from = moveGetFrom(move);
			int to = moveGetTo(move);
			int cPiece = moveGetCapturePiece(move);
			if(from > 32){
				addPiece(position,WHITE_PAWN,from);
				removePiece(position,WHITE_KNIGHT,to);
			}
			else{
				addPiece(position,BLACK_PAWN,from);
				removePiece(position,BLACK_KNIGHT,to);
			}
			if(cPiece != EMPTY) addPiece(position,cPiece,to);
			break;
		}
	}
	position->castling = moveGetCastling(move);
	position->enPassant = moveGetEnPassant(move);
	position->check = moveGetCheck(move);
	position->key = position->historyOfKeys[position->ply];
	return move;
}

int makeNextMove(struct bitboard* position, int moves[], int *index){
	while(moves[*index] != 0) 
		if(makeMove(position, moves[(*index)++])){
			return 1;
		}
	return 0;
}

inline int isRemi(struct bitboard* position){
	if(position->staticMoves[position->ply] < 8) return 0;
	else {
		int i, rp = 0;
		for(i=4; i<=position->staticMoves[position->ply]; i+=2){
			if(position->key == position->historyOfKeys[position->ply-i]){
				rp ++;
				i+=2;
			}
		}
		if(rp == 2) return 1;
		if(position->staticMoves[position->ply] >= 100) return 1;
		return 0;
	}
}

void makeNullMove(struct bitboard* position){
	int move = 0;
	position->historyOfKeys[position->ply] = position->key;	
	position->key ^= position->enPassant;  					
	moveSetEnPassant(move,position->enPassant);				
	moveSetCastling(move,position->castling); 				
	position->enPassant = 0; 								
	changePlayer(position);									
	position->historyOfMoves[position->ply] = move;			
	position->ply ++;										
}

void unmakeNullMove(struct bitboard* position){
	position->ply--;
	int move = position->historyOfMoves[position->ply];
	uChangePlayer(position);
	position->castling = moveGetCastling(move);			
	position->enPassant = moveGetEnPassant(move);		
	position->key = position->historyOfKeys[position->ply];
}

void bitboardPrint(struct bitboard* position){
	char board[72] = {
		'#', ' ', '#', ' ', '#', ' ', '#', ' ', '\n',
		' ', '#', ' ', '#', ' ', '#', ' ', '#', '\n',
		'#', ' ', '#', ' ', '#', ' ', '#', ' ', '\n',
		' ', '#', ' ', '#', ' ', '#', ' ', '#', '\n',
		'#', ' ', '#', ' ', '#', ' ', '#', ' ', '\n',
		' ', '#', ' ', '#', ' ', '#', ' ', '#', '\n',
		'#', ' ', '#', ' ', '#', ' ', '#', ' ', '\n',
		' ', '#', ' ', '#', ' ', '#', ' ', '#', '\0'
	};
	int piece, index;
	for(index = 0; index<64; index++){
		piece = position->types[index];
		if(piece >= PAWN) board[70-rank(index)*9-(7-file(index))] = pieceChar[piece];
	}
	printf("%s\n\n",board);
	if(position->player == WHITE) printf("Player: WHITE\n");
	else printf("Player: BLACK\n");
	printf("Ply:%d\n",position->ply);
	printf("En passant:%d\n",position->enPassant);
	printf("Castling:%d\n",position->castling);
	printf("Check:%d\n",position->check);
	printf("Material:%d\n",position->material[WHITE]+position->material[BLACK]);
	printf("Key:%lld\n",position->key);
	printf("History:\n");
	for(index=0; index<position->ply; index++){
		moveTypeSPrint(board,position->historyOfMoves[index]);
		printf("\t%d.%s\n",index+1,board);
	}
}

int isMat(struct bitboard* position){
	int moves[GENERATOR_MOVES];
	generateMoves(position,moves);
	int index = 0;
	if(!makeNextMove(position,moves,&index)){
		if(isInCheck(position)) return 1;
		return 0;
	}
	unmakeMove(position);
	return 0;
}

int isStaleMat(struct bitboard* position){
	int moves[GENERATOR_MOVES];
	generateMoves(position,moves);
	int index = 0;
	if(!makeNextMove(position,moves,&index)){
		if(!isInCheck(position)) return 1;
		return 0;
	}
	unmakeMove(position);
	return 0;
}

void bitboardSPrint(char stype[],UINT64 x){
	int i, index;
	for(i=0; i<64; i++) stype[i] = '0';
	stype[64] = '\0';
	while(x != 0){
		index = getFirstBitAndClear64(&x);
		stype[63-index] = '1';
	}
}

void bitboardCSPrint(char stype[],UINT64 x){
	int i, index;
	for(i=0; i<71; i++) stype[i] = '0';
	for(i=8; i<71; i+=9) stype[i] = '\n';
	stype[71] = '\0';
	while(x != 0){
		index = getFirstBitAndClear64(&x);
		stype[70-rank(index)*9-(7-file(index))] = '1';
	}
}

int numberOfBits64(UINT64 b){
    int n;
    for (n = 0; b != 0; n++, b = b & (b - 1));
    return n;
}
