/***************************************************************************
 *   Copyright (C) 2005 by Borko Bokovi                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef BITBOARD_H
#define BITBOARD_H

#define rand64() (((UINT64)rand())|(((UINT64)rand())<<16)|(((UINT64)rand())<<32)|(((UINT64)rand())<<46))

typedef long long INT64;

typedef unsigned long long UINT64;

#define ONE64 1ull

#define MAX_PLY 400

struct bitboard{
	UINT64* pawns; 

	UINT64* knights;

	UINT64* bishops;

	UINT64* rooks;

	UINT64* queens;

	UINT64* king;

	UINT64 pieces[14];

	UINT64 freeSquares;

	UINT64 notFreeSquares;

	UINT64 piecesRL90;

	UINT64 piecesRL45;

	UINT64 piecesRR45;

	UINT64 key;

	UINT64 historyOfKeys[MAX_PLY];

	int ply;

	int player;

	int opponent;

	int enPassant;

	int castling;
	
	int isCastle[4];

	int check;

	int types[64];

	int historyOfMoves[MAX_PLY];

	int staticMoves[MAX_PLY];

	int numberOfPieces[2];

	int material[2];
};

void initBitboard(UINT64 aSquaresKeys[14][64], UINT64 aPlayerkey);

void bitboardStartPosition(struct bitboard* position);

void bitboardPrint(struct bitboard* position);

int makeMove(struct bitboard* position, int move);

int unmakeMove(struct bitboard* position);

int makeNextMove(struct bitboard* position, int moves[], int *index);

int makeSMove(struct bitboard* position, char move[]);

void makeSMoves(struct bitboard* position, char moves[]);

inline int isRemi(struct bitboard* position);

inline int isInCheck(struct bitboard* position);

void makeNullMove(struct bitboard* position);

void unmakeNullMove(struct bitboard* position);

void bitboardSetPosition(struct bitboard* position, char fen[]);

int isMat(struct bitboard* position);

int isStaleMat(struct bitboard* position);

void bitboardSPrint(char stype[],UINT64 x);

void bitboardCSPrint(char stype[],UINT64 x);

int numberOfBits64(UINT64 b);

#endif
