/***************************************************************************
 *   Copyright (C) 2005 by Borko Bošković   *
 *   borko.boskovic@uni-mb.si   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <glib.h>
 
 #include "egd.h"
 #include "asm.h"
 #include "piece.h"
 #include "generator.h"
 #include "square.h"
 #include "transpositiontable.h"
 #include "bitboard.h"

inline int bishopsCannotMate(UINT64 bishops){
	int white = 0, black = 0;
	while(bishops){
		if(getFirstBitAndClear64(&bishops) % 2 == 0) white = 1;
		else black = 1;
		if(white && black) return 0;
	}
	return 1;
}

inline int kingsDistance(struct bitboard* position){
	return distance(indexOfFirstBit64(&position->king[WHITE]),
					indexOfFirstBit64(&position->king[BLACK]));
}

int KBBK(struct bitboard * position){
	int loneKingPlayer = position->player;
	if(position->bishops[loneKingPlayer] != 0) loneKingPlayer = position->opponent;

	if(bishopsCannotMate(position->bishops[loneKingPlayer^1])) return 0;

	int score= 11.0*PAWN_VALUE-kingsDistance(position)*(PAWN_VALUE/8.0)-
				cornerDistance(indexOfFirstBit64(&position->king[loneKingPlayer]))*
				(PAWN_VALUE/4.0);
				
	if(loneKingPlayer != position->opponent)
		score = -score;
	return score;
}

int KBNK(struct bitboard * position){
 	int loneKingPlayer = position->player;
	if(position->bishops[loneKingPlayer] != 0) loneKingPlayer = position->opponent;
	
	int king = indexOfFirstBit64(&position->king[loneKingPlayer]);
	int bishop = indexOfFirstBit64(&position->bishops[loneKingPlayer^1]);
	int f = file(bishop); 
	int r = rank(bishop); 
	int color = WHITE;
	int kb_cornerDistance;
	if((r%2 == 0 && f%2 == 0) || (r%2 != 0 && f%2 != 0)) color = BLACK;
	if(color == WHITE) kb_cornerDistance = MIN(distance(king,h1),distance(king,a8));
	else kb_cornerDistance = MIN(distance(king,a1),distance(king,h8));
	int  score = 10.0*PAWN_VALUE - kb_cornerDistance*(PAWN_VALUE/4.0) - 		
				kingsDistance(position)*(PAWN_VALUE/8.0);
	if(loneKingPlayer != position->opponent) score = -score;
	return score;
}

int KRK(struct bitboard* position){
 	int loneKingPlayer = position->player;
	if(position->rooks[loneKingPlayer] != 0) loneKingPlayer = position->opponent;
	
	int king = indexOfFirstBit64(&position->king[loneKingPlayer]);
	int f = file(king); 
	int r = rank(king); 
	int distance = MIN(MIN((7-f),f),MIN((7-r),r));
	int score = 12.0*PAWN_VALUE - distance*(PAWN_VALUE/4.0)-kingsDistance(position)*(PAWN_VALUE/8.0);
	if(loneKingPlayer != position->opponent) score = -score;
	return score;
}

int KQK(struct bitboard* position){
 	int loneKingPlayer = position->player;
	if(position->queens[loneKingPlayer] != 0) loneKingPlayer = position->opponent;
	
	int king = indexOfFirstBit64(&position->king[loneKingPlayer]);
	int f = file(king); 
	int r = rank(king); 
	int distance = MIN(MIN((7-f),f),MIN((7-r),(r)));
	int score = 13.0*PAWN_VALUE - distance*(PAWN_VALUE/4.0)-kingsDistance(position)*(PAWN_VALUE/8.0);
	if(loneKingPlayer != position->opponent) score = -score;
	return score;
}
