/***************************************************************************
 *   Copyright (C) 2005 by Borko Bokovi                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define _GNU_SOURCE
#include <glib.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h> 

#include "engine.h"
#include "search.h"
#include "transpositiontable.h"
#include "timer.h"
#include "generator.h"
#include "piece.h"
#include "uci.h"
#include "bitboard.h"
#include "move.h"
#include "openingbook.h"
#include "evaluator.h"
 
struct bitboard position;

void engineStart(char book[]){
	srand(time(0));
	if (!g_thread_supported ()) g_thread_init (0);
	else {
		fprintf(stderr,"Threads not suported !!!\n");
		exit(1);
	}
	uciInit();
	initEvaluator();
	if(book != 0){
		UINT64 fSquareKeys[14][64];
		UINT64 fPlayerKey, fbooksize;
		int i;
		int tsize = sizeof(UINT64);
		FILE *file = fopen(book,"r");
		if(file == 0){
			fprintf(stderr,"Can't open file: %s\n",book);
			exit(1);
		}
		fread(fSquareKeys,  14*64,  tsize, file);
		fread(&fPlayerKey,  1,  tsize, file);
		initBitboard(fSquareKeys,fPlayerKey);
		fread(&fbooksize, 1, tsize, file);
		UINT64* array = (UINT64 *)calloc(tsize,fbooksize);
		fread(array,  fbooksize,  tsize, file);
		bookCreate((int)fbooksize);
		for(i=0; i<fbooksize; i++) bookPut(i,array[i]);
		free(array);
		fclose(file);
	}
	else initBitboard(0,0);
	ttCreate(64);
	initTimer();
	initEvaluator();
	bitboardStartPosition(&position);
}
	
void engineNewGame(){
}

void engineSetPosition(char parameters[]){
	char token[50];
	uciNextToken(parameters,token);
	if(!strcmp("startpos",token)){
		bitboardStartPosition(&position);
		uciNextToken(parameters,token);
		if(!strcmp("moves",token)) makeSMoves(&position,parameters);
	}
	else if(!strcmp("fen",token)){
		char fen[100];
		fen[0] = '\0';
		uciNextToken(parameters,token);
		while(strcmp(token,"moves") && strcmp(token,"")){
			strcat(fen,token);
			strcat(fen," ");
			uciNextToken(parameters,token);
		}
		bitboardSetPosition(&position,fen);
		if(!strcmp("moves",token)) makeSMoves(&position,parameters);
	}
	else{
		fprintf(stderr,"Error: (engineSetPosition) wrong roken:%s\n",token);
		exit(1);
	}
}

void engineIdentity(FILE* stream){
	fprintf(stream,"id name BBChess 0.99\n");
	fprintf(stream,"id author Borko Boskovic	\n");
	fprintf(stream,"option name Hash type spin default 64 min 1 max 1024\n");
	fprintf(stream,"uciok\n");
	fflush(stream);
}

void engineIsReady(FILE* stream){ 
	fprintf(stream, "readyok\n"); 
	fflush(stream);
}

void engineSetOption(char option[]){
	char token[15];
	uciNextToken(option,token);
	uciNextToken(option,token);
	if(!strcmp(token, "Hash")){
		uciNextToken(option, token);
		if(!strcmp(token,"value")){
			uciNextToken(option,token);
			ttDestroy();
			ttCreate(atoi(token));
		}
	}
}

void engineStartSearching(char parameters[]){
	char token[50];
	uciNextToken(parameters,token);
	if(!strcmp("depth",token)){
		uciNextToken(parameters,token);
		setSearch(atoi(token),0);
	}
	else if(!strcmp("infinite",token)){
		setSearch(MAX_DEPTH,0);
	}
	else if(!strcmp("movetime",token)){
		uciNextToken(parameters,token);
		setSearch(MAX_DEPTH,0);
		stratTimer(atoi(token)*1000);
	}
	else if(!strcmp("mate",token)){
		setSearch(MAX_DEPTH,0);
	}
	else if(!strcmp("nodes",token)){
		uciNextToken(parameters,token);
		setSearch(MAX_DEPTH,atoi(token));
	}
	else{
		int wtime=0, btime=0, winc=0, binc=0, movestogo=40, mytime;
		setSearch(MAX_DEPTH,0);
		while(strcmp(token,"")){
			if(!strcmp("wtime",token)){
				uciNextToken(parameters,token);
				wtime = atoi(token);
			}
			else if(!strcmp("btime",token)){
				uciNextToken(parameters,token);
				btime = atoi(token);
			}
			else if(!strcmp("winc",token)){
				uciNextToken(parameters,token);
				winc = atoi(token);
			}
			else if(!strcmp("binc",token)){
				uciNextToken(parameters,token);
				binc = atoi(token);
			}
			else if(!strcmp("movestogo",token)){
				uciNextToken(parameters,token);
				movestogo = atoi(token);
			}
			uciNextToken(parameters,token);
		}
		if(position.player == WHITE){
			if(movestogo > 5)
				mytime = (int)(winc + (wtime/movestogo)*1.2);
			else
				mytime = (int)((wtime/movestogo)*0.8);
		}
		else{
			if(movestogo > 5)
				mytime = (int)(binc + (btime/movestogo)*1.2);
			else
				mytime = (int)((btime/movestogo)*0.8);
		}
		stratTimer(mytime*1000);
	}
	stratSearch(&position);
}

void engineStopSearching(){
	stopSearch();
}

void engineStop(){
	ttDestroy();
	bookDestroy();
	stopSearch();
	destroyTimer();
}

int engineWaitForSearching(){
	return waitForSearching();
}

void engineInfo(){
	bitboardPrint(&position);
}

struct bitboard* engineGetBitBoard(){
	return &position;
}
