/***************************************************************************
 *   Copyright (C) 2005 by Borko Bokovi                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>

#include "asm.h"
#include "egd.h"
#include "evaluator.h"
#include "piece.h"
#include "generator.h"
#include "transpositiontable.h"
#include "search.h"
#include "square.h"

extern UINT64 attackKnight[64];
extern UINT64 attackBishopL45[64][64];
extern UINT64 attackBishopR45[64][64];
extern UINT64 attackRookL00[64][64];
extern UINT64 attackRookL90[64][64];
extern const int shiftR45[64];
extern const int shiftL45[64];

int mobilityBishopL45[64][64];
int mobilityBishopR45[64][64];
int mobilityRookL00[64][64];
int mobilityRookL90[64][64];
int mobilityKnight8[64];

inline int mobilityR45(int square, UINT64 piecesR45){
	return mobilityBishopR45[square][((piecesR45)>>shiftR45[square])&63];
}

inline int mobilityL45(int square, UINT64 piecesL45){
	return mobilityBishopL45[square][((piecesL45)>>shiftL45[square])&63];
}

inline int mobilityRank(int square, UINT64 pieces){
	return mobilityRookL00[(square)&7][((pieces)>>(((square)&~7)+1))&63];
}

inline int mobilityFile(int square, UINT64 piecesL90){
	return mobilityRookL90[square][((piecesL90)>>((((square)&7)<<3)+1))&63];
}

inline int mobilityBishop(struct bitboard* position, int square){ 
	return mobilityR45(square,position->piecesRR45)+\
			mobilityL45(square,position->piecesRL45);
}

inline int mobilityRook(struct bitboard* position, int square){
	return mobilityRank(square,position->notFreeSquares)+\
			mobilityFile(square,position->piecesRL90);
}

inline int mobilityQueen(struct bitboard* position, int square){
	return mobilityRook(position, square) + mobilityBishop(position, square);
}

inline int mobilityKnight(int square){
	return mobilityKnight8[square];
}


void initEvaluator(){
	initGenerator();
	 int s, b;
	 for(s=0; s<64; s++){
	 	mobilityKnight8[s] = numberOfBits64(attackKnight[s]);
	 	for(b=0; b<64; b++){
			mobilityBishopL45[s][b] = numberOfBits64(attackBishopL45[s][b]);
			mobilityBishopR45[s][b] = numberOfBits64(attackBishopR45[s][b]);
			mobilityRookL00[s][b] = numberOfBits64(attackRookL00[s][b]);
			mobilityRookL90[s][b] = numberOfBits64(attackRookL90[s][b]);
		}
	}
}

inline int evalEndGame(struct bitboard* position){
		int material = position->material[WHITE] - position->material[BLACK];
		int numberOfPieces = position->numberOfPieces[WHITE] + position->numberOfPieces[BLACK];
		switch(numberOfPieces){
			case 2:{ return 0; } //KK
			case 3:{
				switch(material){
					case BISHOP_VALUE 	:
					case -BISHOP_VALUE	:{ return 0; }// KBK
					case KNIGHT_VALUE	:
					case -KNIGHT_VALUE	:{ return 0; } // KNK
					case ROOK_VALUE		:
					case -ROOK_VALUE	:{ return KRK(position); } // KRK
					case QUEEN_VALUE	:
					case -QUEEN_VALUE	:{ return KQK(position); } // KQK
					case PAWN_VALUE		:
					case -PAWN_VALUE	:{ return material; } // KPK
				}
				break;
			}
			case 4:{
				switch(material){
					case(BISHOP_VALUE + BISHOP_VALUE)	:
					case(-(BISHOP_VALUE+BISHOP_VALUE))	:{ return KBBK(position); } // KBBK
					case(BISHOP_VALUE+KNIGHT_VALUE)		:
					case(-(BISHOP_VALUE+KNIGHT_VALUE))	:{ return KBNK(position); } // KBNK
					case(KNIGHT_VALUE+KNIGHT_VALUE)		:
					case(-(KNIGHT_VALUE+KNIGHT_VALUE))	:{ return 0; } //KNNK
					case 0								:{
						if(position->knights[WHITE] != 0) return 0; // KNKN
						if(position->bishops[WHITE] != 0) return 0; // KBKB
					}
					case(KNIGHT_VALUE - BISHOP_VALUE)	:
					case(BISHOP_VALUE - KNIGHT_VALUE)	:{ return 0; } // KBKN
				}
				break;
			}
		}
	return material;
}

int evaluate(struct bitboard* position, int alpha,int beta){
	int numberOfPieces = position->numberOfPieces[WHITE] + position->numberOfPieces[BLACK];
	if(numberOfPieces <= 4)	return evalEndGame(position);
	
	int result = position->material[WHITE] - position->material[BLACK];
	int mobility = 0;
	int from;
	int dist;
	int wkf, bkf, wkr, bkr, pf, pr;
	UINT64 p;

	if(position->player == WHITE){
		if(result - 300 >= beta) return beta;
		if(result + 300 <= alpha) return alpha;
	}else{
		if(-result - 300 >= beta) return beta;
		if(-result + 300 <= alpha) return alpha;
	}
		
	wkf = file(indexOfFirstBit64(&position->king[WHITE]));
	bkf = file(indexOfFirstBit64(&position->king[BLACK]));
	wkr = rank(indexOfFirstBit64(&position->king[WHITE]));
	bkr = rank(indexOfFirstBit64(&position->king[BLACK]));
		
	if((position->castling & 3) == 0){
		if(position->queens[BLACK] != 0 && position->material[BLACK]>2000){
			if(wkf > 5 && (position->rooks[WHITE] & 192) == 0) result += 50;
			else if (wkf < 3 && (position->rooks[WHITE] & 7) == 0) result += 30;
			else result -= 50;
		}

		p = position->queens[WHITE];
		while(p != 0){
			from = getFirstBitAndClear64(&p);
			mobility += mobilityQueen(position, from);
			pf = file(from);
			pr = rank(from);
			result +=  10 - ((ABS(bkf-pf) + ABS(bkr-pr))); 
			if(pf == 6) result += 10;
		}
		p = position->rooks[WHITE];
		while(p != 0){
			from = getFirstBitAndClear64(&p);
		 	mobility += mobilityRook(position, from);
			pf = file(from);
			if((bitboardFile[pf] & position->pawns[WHITE]) == 0){
				result += 10;
				if((bitboardFile[pf] & position->pawns[BLACK]) == 0){
					result += 10;
				}
				dist = ABS(bkf-file(from));
				if(dist <= 1){
					result += 10;
					if(dist == 0) result += 10; 
				}
				if(rank(from) == 6) result += 10;
			}
		}
	}

	if((position->castling & 12) == 0 && numberOfPieces > 13){
		if(position->queens[WHITE] != 0 && position->material[WHITE]>2000){
			if(bkf > 5 && (position->rooks[WHITE] & 13835058055282163712ull) == 0) 
				result -= 50;
			else if(bkf < 3 && (position->rooks[WHITE] & 216172782113783808ull) == 0)
				result -= 30;
			else result += 50;
		}
		
		p = position->queens[BLACK];
		while(p != 0){
			from = getFirstBitAndClear64(&p);
			mobility -= mobilityQueen(position, from);
			pf = file(from);
			pr = rank(from);
			result -=  10 - ((ABS(wkf-pf) + ABS(wkr-pr))); 
			if(pf == 1) result -= 10;
		}
		p = position->rooks[BLACK];
		while(p != 0){
			from = getFirstBitAndClear64(&p);
			mobility -= mobilityRook(position,from);
			pf = file(from);
			if((bitboardFile[pf] & position->pawns[BLACK]) == 0){
				result -= 10;
				if((bitboardFile[pf] & position->pawns[WHITE]) == 0){
					result -= 10;
				}
				dist = ABS(wkf-file(from));
				if(dist <= 1){
					result -= 10;
					if(dist == 0) result -= 10; 
				}
				if(rank(from) == 1) result -= 10;
			}
		}
	}
	
	p = position->bishops[WHITE];
	while(p != 0) mobility += mobilityBishop(position, getFirstBitAndClear64(&p));
	
	p = position->bishops[BLACK];
	while(p != 0) mobility -= mobilityBishop(position, getFirstBitAndClear64(&p));
	
	p = position->knights[WHITE];
	while(p != 0) mobility +=mobilityKnight(getFirstBitAndClear64(&p));
		
	p = position->knights[BLACK];
	while(p != 0) mobility -= mobilityKnight(getFirstBitAndClear64(&p));
	
	p = position->pawns[WHITE];
	mobility += numberOfBits64(shiftUp(p) & position->freeSquares);
	result -= numberOfBits64((p & (p << 8)) | (p & (p << 16)) | (p & (p << 24)))<<6;
	result += numberOfBits64(p & rank5)<<2;
	result += numberOfBits64(p & rank6)<<4;
	result += numberOfBits64(p & rank7)<<6;

	p = position->pawns[BLACK];
	mobility -= numberOfBits64(shiftDown(p) & position->freeSquares);
	result += numberOfBits64((p & (p >> 8)) | (p & (p >> 16)) | (p & (p >> 24)))<<6;
	result -= numberOfBits64(p & rank4)<<2;
	result -= numberOfBits64(p & rank3)<<4;
	result -= numberOfBits64(p & rank2)<<6;
	
	result += (mobility<<2);
	
	if(position->player == WHITE) return result;
	else return -result;
}
