/***************************************************************************
 *   Copyright (C) 2005 by Borko Bokovi                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MOVE_H
#define MOVE_H

#include "piece.h"
#include "square.h"

/**
 * \file move.h
 * <pre>
 * 00000000000000000000000000111111 - to  
 * 00000000000000000000111111000000 - from
 * 00000000000000001111000000000000 - type
 * 00000000000011110000000000000000 - capture piece
 * 00000000111100000000000000000000 - castling 
 * 00111111000000000000000000000000 - en passant
 * 01000000000000000000000000000000 - ckeck
 * </pre>
 */

#define NORMAL_MOVE 4096

#define CAPTURE_MOVE 8192

#define ENPASSANT_MOVE 12288

#define WHITE_CASTLING_KING_SIDE 16646

#define WHITE_CASTLING_QUEEN_SIDE 20738

#define BLACK_CASTLING_KING_SIDE 28478

#define BLACK_CASTLING_QUEEN_SIDE 32570

#define PROMOTION_QUEEN 32768

#define PROMOTION_ROOK 36864

#define PROMOTION_BISHOP 40960

#define PROMOTION_KNIGHT 45056

#define TYPE_STATUS 61440

#define CAPTURE_STATUS -983041

#define CASTLING_STATUS -15728641

#define ENPASSANT_STATUS -1056964609

#define CHECK_STATUS -1073741825

#define CHECK_TEST 1073741824

#define generateNormalMove(from,to) (NORMAL_MOVE|to|(from<<6))

#define generateCaptureMove(from,to) (CAPTURE_MOVE|to|(from<<6))

#define generateEnPassantMove(from,to) (ENPASSANT_MOVE|to| (from<<6))

#define generateWhiteCastlingKingSideMove() WHITE_CASTLING_KING_SIDE

#define generateWhiteCastlingQeenSideMove() WHITE_CASTLING_QUEEN_SIDE

#define generateBlackCastlingKingSideMove() BLACK_CASTLING_KING_SIDE

#define generateBlackCastlingQeenSideMove() BLACK_CASTLING_QUEEN_SIDE

#define generatePromotionQueenMove(from,to) (PROMOTION_QUEEN|to|(from<<6))

#define generatePromotionRookMove(from,to) (PROMOTION_ROOK|to|(from<<6))

#define generatePromotionBishopMove(from,to) (PROMOTION_BISHOP|to|(from<<6))

#define generatePromotionKnightMove(from,to) (PROMOTION_KNIGHT|to|(from<<6))

#define moveSetCapturePiece(move,piece) move&=CAPTURE_STATUS;move|=piece<<16;

#define moveSetCastling(move,castling) move&=CASTLING_STATUS;move|=castling<<20

#define moveSetEnPassant(move,enPassant) move&=ENPASSANT_STATUS;move|=enPassant<<24

#define moveSetCheck(move,check) move&=CHECK_STATUS;move|=check<<30

#define moveGetCapturePiece(move) ((move>>16)&15)

#define moveGetType(move) (move&TYPE_STATUS)

#define moveGetFrom(move) ((move>>6)&63)

#define moveGetTo(move) (move&63)

#define moveGetCastling(move) ((move>>20)&15)

#define moveGetEnPassant(move) ((move>>24)&63)

#define moveGetCheck(move) ((move&CHECK_TEST)>>30)

void moveSPrint(char smove[], int move);

void movePrint(int move);

void moveTypeSPrint(char smove[], int move);

void moveTypePrint(int move);

void printMoves(int moves[]);

#endif
