/***************************************************************************
 *   Copyright (C) 2005 by Borko Bokovi                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define timersub(a, b, result)                                               \
  do {                                                                        \
    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;                             \
    (result)->tv_usec = (a)->tv_usec - (b)->tv_usec;                          \
    if ((result)->tv_usec < 0) {                                              \
      --(result)->tv_sec;                                                     \
      (result)->tv_usec += 1000000;                                           \
    }                                                                         \
  } while (0)


#include <sys/time.h>
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "search.h"
#include "bitboard.h"
#include "uci.h"
#include "generator.h"
#include "transpositiontable.h"
#include "engine.h"
#include "evaluator.h"
#include "move.h"
#include "openingbook.h"
#include "timer.h"
#include "egd.h"

unsigned int nodes;
unsigned int maxNodes;
unsigned short depth;
unsigned short maxDepth;
int searchStop;
int INF = 5000;
int alphaBeta(struct bitboard* pos, int alpha, int beta, int depth, int nullMove);
int quiescent(struct bitboard* pos, int alpha, int beta);
gpointer runSearchFunction(gpointer data);
GThread* searchThread = 0;
int history[64][64];
int pv[MAX_PV];
void setSearch(unsigned short aMaxDepth, unsigned int aMaxNodes){
	maxDepth = aMaxDepth;
	maxNodes = aMaxNodes;
}

gpointer runSearchFunction(gpointer data){
	struct bitboard* pos = (struct bitboard*) data;
	nodes = 0;
	int i, j, up = 0, down = 0, window=3, evaluate=0, depth = 1;
	int alpha = evaluate-window, beta=evaluate+window;
	for(i=0; i<MAX_PV; i++) pv[i] = 0;
	pv[0] = bookSearch(pos);
	if(pv[0] != 0){
		uciWriteBestMove(pv[0]);
		stopTimer();
		return (gpointer)pv[0];
	}
	GTimeVal startTime;
	GTimeVal endTime;
	GTimeVal workTime;
	g_get_current_time(&startTime);
	for(i=0; i<64; i++){
		for(j=0; j<64; j++){
			history[i][j] = 0;
		}
	}
	ttReset();
	while (depth <= maxDepth){
		evaluate = alphaBeta(pos, alpha, beta, depth, 1);
		if(searchStop || (maxNodes != 0 && nodes > maxNodes)) break;
		g_get_current_time(&endTime);
		timersub(&endTime, &startTime, &workTime);
		ttPrincipalVariant(pos,pv);
		uciWriteInfo(evaluate, depth, nodes, workTime, pv,ttUsage());
		if(evaluate == INF || evaluate == -INF) break;
		if (evaluate <= alpha){
			down = 1;
			if(up != down){
				window += window;
				beta = evaluate+1;
				alpha = evaluate - window;
				if(alpha < -INF) alpha = -INF-1;
				continue;
			}
		}
		if (evaluate >= beta){
			up = 1;
			if(up != down){
			window += window;
				alpha = evaluate-1;
				beta = evaluate + window;
				if(beta > INF) beta = INF+1;
				continue;
			}
      	}
		window = 3;
		alpha = evaluate - window;
		beta = evaluate + window;
		up = down = 0;
		depth++;
	}
	uciWriteBestMove(pv[0]);
	stopTimer();
	return (gpointer)pv[0];
}

int alphaBeta(struct bitboard* pos, int alpha, int beta, int depth, int nullMove){
	int bestMove=0, lastMove=0, value, index=0, extend = 0, fscore, fprune=0, fmax=0;
	int mat_balance;
	nodes++;
	if(searchStop || (maxNodes != 0 && nodes > maxNodes)) return -INF;
	if(isRemi(pos))	return 0;
	struct TranspositionTable* tMove = ttFind(pos->key);
	if(tMove->depth >= depth){
		if(tMove->flag == EXACT) return tMove->evaluate;
		else if(tMove->flag == ALPHA && tMove->evaluate > alpha){
			bestMove = tMove->move;
			alpha = tMove->evaluate;
		}
		else if(tMove->flag == BETA && tMove->evaluate < beta)
			beta = tMove->evaluate;
		if(alpha >= beta){
			if(tMove->move != 0){
				makeMove(pos,tMove->move);
				if(beta <= 0 && isRemi(pos)){
					unmakeMove(pos);
					return 0;
				}
				unmakeMove(pos);
			}
			return alpha;
		}
	}
	int ttMove = tMove->move;
	if(isInCheck(pos)){
		depth ++;
		extend = 1;
	}

	if(depth < 1) return quiescent(pos,alpha, beta);
	
	if (depth > 2 && !nullMove && !extend && 
		pos->numberOfPieces[WHITE] > 3 && pos->numberOfPieces[BLACK] > 3) {
		makeNullMove(pos);
		if(depth > 6) value = -alphaBeta(pos,-beta, -beta + 1, depth - 4, 1);
		else value = -alphaBeta(pos,-beta, -beta + 1, depth - 3, 1);
		unmakeNullMove(pos);
		if(value >= beta) return value;
	}

	if(ttMove == 0 && depth > 3 && depth >= ttDepth(pos->key)){
		alphaBeta(pos, alpha, beta, depth-3, 1);
		ttMove = (ttFind(pos->key))->move;
	}

	if(ttMove != 0) {
		makeMove(pos,ttMove);
		value = -alphaBeta(pos,-beta, -alpha, depth - 1, 0);
		lastMove = unmakeMove(pos);
		if(value > alpha) {
			if(value >= beta){
				ttRecord(pos->key, depth, value, ALPHA, lastMove);
				return value;
			}
			alpha = value;
			bestMove = lastMove;
		}
	}

	int moves[GENERATOR_MOVES];
	int endgame = 0;
	generateDynamicMoves(pos,moves);
	if(!makeNextMove(pos,moves,&index))endgame = 1;
	else{
		do{
			value = -alphaBeta(pos,-beta,-alpha , depth-1, 0);
			lastMove = unmakeMove(pos);
			if(value > alpha){
				if(value >= beta){
					ttRecord(pos->key,depth,value,ALPHA,lastMove);
					return value;
				}
				alpha = value;
				bestMove = lastMove;
			}
		}while(makeNextMove(pos,moves,&index));
	}

	mat_balance = pos->material[pos->player] - pos->material[pos->opponent];
	fscore = mat_balance + 900;
	if(!extend && depth == 3 && fscore <= alpha && pos->numberOfPieces[pos->opponent] > 3)
		depth = 2;

	fscore -= 400;
	if(!extend && depth == 2 && fscore <= alpha){
		fprune = 1;
		fmax = fscore;
	}
	fscore -= 300;
	if(!extend && depth == 1 && fscore <= alpha){
		fprune = 1;
		fmax = fscore;
	}
	
	index = 0;
	generateStaticMoves(pos,moves,history);
	if(!makeNextMove(pos,moves,&index)){
		if(endgame == 1 && ttMove ==0){
			if(isInCheck(pos)) return -INF;
			return 0;
		}
	}
	else{
		do{
			if(!fprune || isInCheck(pos) ||  fmax > alpha){
				value = -alphaBeta(pos,-beta,-alpha , depth-1, 0);
				lastMove = unmakeMove(pos);
				if(value > alpha){
					if(value >= beta){
						history[moveGetFrom(lastMove)][moveGetTo(lastMove)] += 1<<depth;
						ttRecord(pos->key,depth,value,ALPHA,lastMove);
						return value;
					}
					alpha = value;
					bestMove = lastMove;
				}
			}
			else{
				unmakeMove(pos);
				nodes++;
			}
		}while(makeNextMove(pos,moves,&index));
	}
	if(bestMove != 0) ttRecord(pos->key,depth,alpha,EXACT,bestMove);
	else ttRecord(pos->key, depth, alpha, BETA, 0);
	return alpha;
}


int quiescent(struct bitboard* pos, int alpha, int beta){
	int value, index=0;
	nodes ++;
	if(searchStop || (maxNodes != 0 && nodes > maxNodes)) return -INF;
	value = evaluate(pos, alpha, beta);
	if(value > alpha){
		if (value >= beta) return value;
		alpha = value;
	}
	int moves[GENERATOR_MOVES];
	generateDynamicMoves(pos,moves);
	while(makeNextMove(pos,moves,&index)){
		value = -quiescent(pos,-beta,-alpha);
		unmakeMove(pos);
		if(value > alpha){
			if(value >= beta) return value;
			alpha = value;
      	}
	}
	return alpha;
}

void stratSearch(struct bitboard* position){
	searchStop = 0;
	searchThread = g_thread_create(runSearchFunction,(gpointer)position,1,0);
}

void stopSearch(){
	searchStop = 1;
	if(searchThread != 0) g_thread_join(searchThread);
	searchThread = 0;
}

void stopSearchSignal(){
	searchStop = 1;
}

int waitForSearching(){
	int result = 0;
	if(searchThread != 0){
		result = (int)g_thread_join(searchThread);
		searchThread = 0;
		return result;
	}
	return 0;
}
