/***************************************************************************
 *   Copyright (C) 2005 by Borko Bokovi                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include <stdlib.h>
#include "transpositiontable.h"
 
unsigned int ttSize;
unsigned int ttFree;
struct TranspositionTable* ttable = 0;
struct TranspositionTable ttNull = { 0, -1, 0, 0, 0};

void ttCreate(unsigned int kSize){
	int size = sizeof(struct TranspositionTable);
	ttSize = (kSize * 1048576)/size;
	ttFree = ttSize;
	ttable = (struct TranspositionTable*)calloc(ttSize,size);
}


void ttDestroy(){
	if(ttable != 0) free((void*)ttable);
}

int ttDepth(UINT64 key){
	return ttable[key% ttSize].depth;
}

struct TranspositionTable* ttFind(UINT64 key){
	struct TranspositionTable* ttMove = &ttable[key% ttSize];
	if(ttMove->key == key) return ttMove;
	return &ttNull;
}

void ttRecord(UINT64 key,int depth, int evaluate, int flag, int move){
	struct TranspositionTable* ttMove = &ttable[key % ttSize];
	if(ttMove->depth <= depth){
		if(ttMove->depth < 0) ttFree --;
    	ttMove->depth = depth;
		ttMove->key = key;
		ttMove->evaluate = evaluate;
		ttMove->flag = flag;
		ttMove->move = move;
	}
}

int ttUsage(){
	return (int)(((ttSize-ttFree)/(float)ttSize)*1000);
}

void ttReset(){
	int i;
	for(i=0; i<ttSize; i++) ttable[i].depth = -1;
	ttFree = ttSize;
}

void ttPrincipalVariant(struct bitboard* position, int ttPV[]){
	int i = 0, move;
	do{
		move = (ttFind(position->key))->move;
		if(move == 0 && i == 0) return;
		ttPV[i] = move;
		if(move != 0){
			makeMove(position,move);
			i++;
		}
	}while(move != 0 && i<MAX_PV-1);
	ttPV[i] = 0;
	for(;i>0; i--) unmakeMove(position);
}
