/***************************************************************************
 *   Copyright (C) 2005 by Borko Bokovi                                  *
 *   borko.boskovic@uni-mb.si                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
 
#include "engine.h"
#include "uci.h"
#include "move.h"

FILE* ucistream;
struct uciinfo ucilastinfo;

void uciNextToken(char line[], char token[]){
	token[0] = '\0';
	if(sscanf(line,"%s",token) == -1) return;
	int i = strlen(token);
	while(line[i] == ' ') i++;
	strcpy(line,line+i);
}

void uciInit(){
	ucistream = stdout;
}

void uciRun(char book[]){
	char line[5000];
	char token[50];
	engineStart(book);
	while (1){
		fgets(line,5000,stdin);
		uciNextToken(line,token);
		if(!strcmp("stop",token)){
			engineStopSearching();
			continue;
		}
		if(!strcmp("go",token)){
			engineWaitForSearching();
			engineStartSearching(line);
			continue;
		}
		if(!strcmp("uci",token)){
			engineIdentity(ucistream);
			continue;
		}
		if(!strcmp("ucinewgame",token)){
			engineNewGame();
			continue;
		}
		if(!strcmp("quit",token)){
			engineStop();
			break;
		}
		if(!strcmp("isready",token)){
			engineIsReady(ucistream);
			continue;
		}
		if(!strcmp("position",token)){
			engineSetPosition(line);
			continue;
		}
		if(!strcmp("setoption",token)){
			engineSetOption(line);
			continue;
		}
		if(!strcmp("",token)){ 
			continue; 
		}
		fprintf(ucistream, "UCI worning:%s\n",token);
	}
}

void uciWriteInfo(int eval, short depth, unsigned int nodes, GTimeVal time, int pv[], int hash){
	unsigned int nps = 0;
	float sectime = (time.tv_sec * 1000000ull + time.tv_usec)/1000000.0;
	char smove[6];
	if(sectime > 0) nps = (int)(nodes/sectime);
	fprintf(ucistream,"info score cp %d depth %d nodes %u ",eval,depth,nodes);
	fprintf(ucistream,"nps %d time %f hashfull %d pv ",nps,sectime,hash);
	if(ucilastinfo.bm != pv[0]) ucilastinfo.bmTime = sectime;
	if(pv[0] == 0) ucilastinfo.bmTime = 0;
	ucilastinfo.depth = depth;
	ucilastinfo.nodes = nodes;
	ucilastinfo.time = sectime;
	ucilastinfo.hash = hash;
	ucilastinfo.bm = pv[0];
	nps=0;
	while(pv[nps] != 0){
		moveSPrint(smove,pv[nps]);
		fprintf(ucistream,"%s ",smove);
		nps++;
	}
	fprintf(ucistream,"\n");
	fflush(ucistream);
}

void uciWriteBestMove(int move){
	char smove[6];
	moveSPrint(smove,move);
	fprintf(ucistream,"bestmove %s\n",smove);
	fflush(ucistream);
}

struct uciinfo* uciLastInfo(){
	return &ucilastinfo;
}

void uciSetStream(FILE* stream){
	ucistream = stream;
}
